/* Canonicalize floating-point representation.
   Copyright (C) 2016-2024 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */

#include <math.h>

int
M_DECL_FUNC (__canonicalize) (FLOAT *cx, const FLOAT *x)
{
  FLOAT val = *x;
  /* For all binary formats supported by glibc, iscanonical only fails
     if the representation is not a valid representation of the type,
     so the only work to do is for signaling NaNs.  */
  if (!iscanonical (val))
    return 1;
  if (issignaling (val))
    *cx = val + val;
  else
    *cx = val;
  return 0;
}

declare_mgen_alias (__canonicalize, canonicalize)
