#define HAVE_NETASH_ASH_H
#define HAVE_NETATALK_AT_H
#define HAVE_NETAX25_AX25_H
#define HAVE_NETECONET_EC_H
#define HAVE_NETIPX_IPX_H
#define HAVE_NETPACKET_PACKET_H
#define HAVE_NETROSE_ROSE_H

#include <socket/sa_len.c>
